/**
 * @file main.c
 *
 * @brief TFT LCD demo main
 *
 */

/**
 *  Copyright 2022-2024 Beijing WinnerMicroelectronics Co.,Ltd.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

#include <stdio.h>
#include "wmsdk_config.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

#include "wm_drv_gpio.h"
#include "wm_drv_tft_lcd.h"
#include "wm_drv_sdh_spi.h"
#include "wm_utils.h"

#include "wm_drv_timer.h"
#include "lvgl.h"
#include "wm_lv_port_disp.h"

#define LOG_TAG "lvgl_example"
#include "wm_log.h"

#include "ui/ui.h"

#define WM_LVGL_TASK_STACK             (1024)
#define WM_LVGL_TASK_PRIO              (configMAX_PRIORITIES - 8)

/* the controller device name which used by LCD, it must same as the device name defined in device table*/
#define LCD_SPI_CONTROLLER_DEVICE_NAME "sdspi"

/* LVGL tick period uint */
#define LV_TICK_PERIOD_MS              (1)

/* CUSTOM OPTION: show winner micro's logo */
#define LV_USE_CUST_WINNER_MICRO_LOGO  (1)

#if LV_USE_CUST_WINNER_MICRO_LOGO
/* include the logo array generated by LVGL image converter */
#include "./assets/winner_micro_logo_120x120.c"
#endif

/* Hardware Peripheral Driver Inititalization */
static void wm_lcd_init(void)
{
    int ret                  = WM_ERR_FAILED;
    wm_device_t *dev         = NULL;
    wm_lcd_capabilitys_t cap = { 0 };

    /* init gpio driver */
    dev = wm_dt_get_device_by_name("gpio");
    if ((!dev) || (dev->state != WM_DEV_ST_INITED)) {
        dev = wm_drv_gpio_init("gpio");
    }
    if (!dev || dev->state != WM_DEV_ST_INITED) {
        wm_log_error("init gpio fail.");
    }

    /*TODO: add more initial methods for other interfaces(like RGB/MIPI...) in future on new chip*/
    /*use sdio(spi mode) for lcd demo as it's support high speed */
    dev = wm_drv_sdh_spi_init(LCD_SPI_CONTROLLER_DEVICE_NAME);
    if (dev == NULL) {
        wm_log_error("init sdspi fail.");
    }

    /* initial the lcd device, and use the same device name which defined in device table*/
    dev = wm_drv_tft_lcd_init(WM_LVGL_LCD_MODULE_NAME);
    if (dev == NULL) {
        wm_log_error("init lcd fail.");
    }

    /* turn on the backlight*/
    ret = wm_drv_tft_lcd_set_backlight(dev, true);
    if (ret != WM_ERR_SUCCESS) {
        wm_log_error("lcd bl set fail.");
    }

    /* show LCD capability */
    wm_drv_tft_lcd_get_capability(dev, &cap);
    wm_log_info("LCD x_resolution = %d", cap.x_resolution);
    wm_log_info("LCD y_resolution = %d", cap.y_resolution);
    wm_log_info("LCD rotation = %d\n", cap.rotation);
}

/* LVGL Timer Porting */
static void wm_lv_tick_timer_irq(void *arg)
{
    lv_tick_inc(LV_TICK_PERIOD_MS);
}

void wm_lv_create_tick(void)
{
    int err                      = 0;
    wm_device_t *timer0_dev      = NULL;
    wm_drv_timer_cfg_t timer_cfg = { 0 };

    if (NULL == (timer0_dev = wm_drv_timer_init("timer0"))) {
        wm_log_error("timer init failed");
    }

    if (WM_ERR_SUCCESS != (err = wm_drv_timer_register_callback(timer0_dev, wm_lv_tick_timer_irq, timer0_dev))) {
        wm_log_error("timer register err");
    }

    timer_cfg.unit        = WM_HAL_TIMER_UNIT_MS;
    timer_cfg.auto_reload = true;
    timer_cfg.period      = LV_TICK_PERIOD_MS;
    wm_drv_timer_start(timer0_dev, timer_cfg);
}

/* LVGL Task Related */
#if LV_USE_DEMO_BENCHMARK
static void on_benchmark_finished(void)
{
    disp_enable_update();
}
#endif

#if LV_USE_LOG
void wm_lv_log_cb(const char *buf)
{
    wm_log_info("%s", buf);
}
#endif

#if LV_USE_CUST_WINNER_MICRO_LOGO
void app_show_company_logo(void)
{
    /* show Winner-Micro LOGO */
    lv_obj_set_style_bg_color(lv_scr_act(), lv_color_make(255, 255, 255), LV_PART_MAIN);
    lv_obj_t *img = lv_img_create(lv_scr_act());
    lv_obj_align(img, LV_ALIGN_CENTER, 0, 0);
    lv_img_set_src(img, &winner_micro_logo_120x120);

    /* show logo right now */
    lv_obj_invalidate(lv_scr_act());
    lv_refr_now(NULL);

    /* keep the logo displayed for a certain period of time */
    vTaskDelay(pdMS_TO_TICKS(2000));
    lv_obj_clean(lv_scr_act());
}
#endif

static void wm_lv_task_entry(void *arg)
{
    wm_lcd_init();

#if LV_USE_LOG
    lv_log_register_print_cb(wm_lv_log_cb);
#endif

    lv_init();

    wm_lv_create_tick();

    lv_port_disp_init();

#if LV_USE_CUST_WINNER_MICRO_LOGO
    app_show_company_logo();
#endif

// /* Enable one demo scenario in lv_confs.h at a time to prevent conflicts */
// #if LV_USE_DEMO_BENCHMARK
//     lv_demo_benchmark_set_finished_cb(&on_benchmark_finished);

//     lv_demo_benchmark_set_max_speed(true);

//     lv_demo_benchmark();
// #endif

// #if LV_USE_DEMO_STRESS
//     lv_demo_stress();
// #endif

// #if LV_USE_DEMO_MUSIC
//     lv_demo_music();
// #endif

// #if LV_USE_DEMO_WIDGETS
//     lv_demo_widgets();
// #endif

    ui_init();

    while (1) {
        lv_task_handler();
        vTaskDelay(pdMS_TO_TICKS(10));
    }

    /* deinit once loop break */
    vTaskDelete(NULL);
}

int main(void)
{
    xTaskCreate(wm_lv_task_entry, "wm_lv_task", WM_LVGL_TASK_STACK, NULL, WM_LVGL_TASK_PRIO, NULL);

    extern int user_clock(void);
    user_clock();

    extern int user_weather(void);
    user_weather();
    
    return 0;
}
